﻿using System;
using System.Threading;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Articles.ThreadingTutorial.Util;

namespace Articles.ThreadingTutorial.Sample2
{
    public class Sample2Console
    {
        private static EventWaitHandle s_eventWaitHandle;
        private static int s_fileCounter;

        /// <summary>
        /// Runs this sample.
        /// </summary>
        public static void Run()
        {
            ConsoleManager.Show();
            try
            {
                Console.WriteLine("ThreadPool Example");

                Console.Write("Press Enter.. ");
                Console.ReadLine();

                Console.WriteLine();
                Console.WriteLine("------------------- Loading in one thread");

                //Loading 5 files
                LoadFile("file1.xml");
                LoadFile("file2.xml");
                LoadFile("file3.xml");
                LoadFile("file4.xml");
                LoadFile("file5.xml");

                Console.Write("Press Enter.. ");
                Console.ReadLine();

                Console.WriteLine();
                Console.WriteLine("------------------- Loading using ThreadPool");

                //Loading 5 files
                s_eventWaitHandle = new EventWaitHandle(false, EventResetMode.ManualReset);
                s_fileCounter = 0;
                ThreadPool.QueueUserWorkItem(new WaitCallback(LoadFile), "file1.xml");
                ThreadPool.QueueUserWorkItem(new WaitCallback(LoadFile), "file2.xml");
                ThreadPool.QueueUserWorkItem(new WaitCallback(LoadFile), "file3.xml");
                ThreadPool.QueueUserWorkItem(new WaitCallback(LoadFile), "file4.xml");
                ThreadPool.QueueUserWorkItem(new WaitCallback(LoadFile), "file5.xml");
                s_eventWaitHandle.WaitOne();

                Console.WriteLine("All work finished!");
                Console.Write("Press Enter.. ");
                Console.ReadLine();
            }
            finally
            {
                ConsoleManager.Hide();
            }
        }

        /// <summary>
        /// Simulate loading of a file.
        /// </summary>
        private static void LoadFile(object argument)
        {
            Console.WriteLine("[" + DateTime.Now.ToLongTimeString() + "] Starting loading of file " + argument.ToString());
            
            //Simulate loading
            Thread.Sleep(1000);

            Console.WriteLine("[" + DateTime.Now.ToLongTimeString() + "] Finished loading of file " + argument.ToString());

            s_fileCounter++;
            if ((s_fileCounter >= 5) && (s_eventWaitHandle != null)) { s_eventWaitHandle.Set(); }
        }
    }
}
